/************************************************************************/
/*                                                                      */
/*              Copyright 1991, CSys Labs Incorporated,                 */      
/*                      238 So. Hillview Dr.                            */
/*                      Milpitas, Ca. 95035                             */
/*                                                                      */
/************************************************************************/

/* TVIOS has all the Touch Vision I/O routines in serial format */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <conio.h>
#include <time.h>
#include <dos.h>
#include <bios.h>
#include <csysdefs.h>
#include <tvios.h>           
#include <sserver.h>

int ModuleType = TVM24128_TYPE; /* Stores module type set by CheckModuleType */

extern FILE * pFile;

static STRING BuffA[200];       /* Buffer used for TX communication */
extern bool bTXBusy;
bool Verbose = 0;               /* Turns on listing of serial commands */
extern bool bKPS;               /* KPS signal */
/*-------------------------------------------------------------------------*/
/*These routines are the low level communications for the serial link. */

/* ssprintf sends the contents of sBuff out to the serial interface */
void ssprintf(STRING * sBuff,STRING * pFormat,...)
{
    va_list argptr;
    va_start(argptr,pFormat);
    while(bTXBusy);
    vsprintf(sBuff,pFormat,argptr);
    SendString(sBuff);
    if(Verbose)printf("%s",sBuff);
    va_end(argptr);
}

/* yylex does the lexical analisis of the data returned from the TVM.  It
returns a token specifying the return type and puts the lexeme into the
buffer "text" (actual string data returned is the lexeme).  A space, \n or ,
mark the end of the lexeme.

        Lexeme          Tokenvalue returned     Token definition
        ______          ___________________     ______
        $<number>       0                       NUMERIC
        #<cmd string>>  1                       ERROR
        @E<string>      2                       ECHO
        @K<bool>        3                       KEYPRESSED
        &<number>       4                       KEYCODE
        NULL TOKEN      -1                      NTOK

*/

STRING text[80];                /* lexeme buffer */
STRING text2[80];               /* Echo lexeme buffer */

/* Static vars used in yylex */
static int yystate = -1;        /* accept state */
static uint sMark = 0;          /* Lexeme Start index mark */
static uint Next = 0;           /* Net buffer index mark */

int yylex()
{
    char yynext;                /* Storage for next character */
    uint yytoken;               /* Temp token storage */
    STRING * pText;             /* Pointer to lex text buffer */
    uint i;
    uint index;                 /* Temporary indexer */
    if(RBIn - Next <= 0)return NTOK;    /* No data available */
    yynext = RBuffer[Next & RBMASK];    /* Get next data */
#ifdef NODEF
    printf("<%c>",yynext);
#endif
    /* Get yystate */
    if(yystate == -1)           /* No accept state yet */
    {
        switch(yynext)
        {
            case '$':           /* Found NUMERIC */
                yystate = NUMERIC;
                break;
            case '#':           /* Found ERROR */
                yystate = ERROR;
                break;
            case '@':           /* Found serial interface command */
                yystate = SIC;
                break;
            case '&':           /* Found KEYPRESSED */
                yystate = KEYPRESSED;
                break;
            default:
                sMark = ++Next; /* Advance past character */
                return NTOK;
        }
    }
    else if(yystate == SIC)     /* SIC State */
    {
        if(yynext == 'K')yystate = KEYPRESSED;
        else if(yynext == 'E')yystate = ECHO;
        else
        {
            sMark = Next;              /* Move sMark past character */
            yystate = -1;
            return NTOK;
        }
    }
    else                                /* Had accept state */
    {
        switch(yynext)
        {
            case ' ':                   /* Found terminator, transfer lexeme */
            case '\n':
            case ',':
                yytoken = yystate;      /* Save state for token */
                yystate = -1;           /* Reset state */
                pText = text;           /* Set pointer */
                index = sMark;          /* Set index to get data */
                for(i=0;i<(Next - sMark);i++)/* Transfer lexeme */
                {
                    *pText++ = RBuffer[index++ & RBMASK];
                }
                *pText = 0;             /* Set null terminator */
                sMark = ++Next;         /* Advance indexers */
                if(Verbose)printf("[%s]",text);
                return yytoken;         /* Return token value */
        }
    }
    Next++;
    return NTOK;
}

/* Echo sends an echo command.  It then waits for the echo
to come back.  If something is returned before the echo, it returns this
token.  Lexeme is returned in text2 in case echo was precedded by something.
If the echo is returned, it returns ECHO token.   Echo
automatically selects a unique echo string to send.  It changes
this string each time.  */

uint Echo()
{
    STRING EchoS[30];
    STRING temp[20];
    static uint Numb = 0;
    int i = -1;
    int tok = -1;
    /* Get a random number in a string */
    sprintf(EchoS,"@E%02d",Numb++%100); /* Form command and send */
    ssprintf(BuffA,"%s\n",EchoS);       /* Send string with \n at end */
    while(1)
    {
        while((i = yylex()) == NTOK);   /* Get token */
        if(tok == -1)
        {
            strcpy(text2,text);         /* Copy lexeme value into text2 */
            tok = i;                    /* Initial assignment */
        }
        if(i == ECHO)                   /* Loop untill ECHO is a exact match */
        {
            if(!strcmp(text,EchoS))return tok;/* Return previous tok value */
        }
        else
        {
            tok = i;                    /* Save token value */
            strcpy(text2,text);         /* Copy lexeme value into text2 */
        }
    }
}

/* SetLowThreshold sets the module low threshold point */
void SetLowThreshold(uint Threshold)
{
    ssprintf(BuffA,"@L%d\n",Threshold);
}

/* SetHighThreshold sets the module low threshold point */
void SetHighThreshold(uint Threshold)
{
    ssprintf(BuffA,"@H%d\n",Threshold);
}

/* EnableXON enables the XON/XOFF software hand shaking */
void EnableXON()
{
    ssprintf(BuffA,"@X\n");    
}

/* ResetXON resets the XON/XOFF software hand shaking */
void ResetXON()
{
    ssprintf(BuffA,"@R\n");
}
/*-------------------------------------------------------------------------*/

/* CheckKEYPRESS tests to see if a key has been pressed.  It return 1 if 
set.  It return 0 if not set. */
bool CheckKEYPRESS(void)
{
    if(bKPS)
    {
        bKPS = 0;
        return 1;
    }
    return 0;
#ifdef NODEF
    STRING * pS;
    int i;
    ssprintf(BuffA,"\n@K ");
    while(yylex() != KEYPRESSED);       /* Wait for valid token */
    return atoi(&text[2]);
#endif
}
/* GetData gets char data from TVM serial port. */
/* return 0 for successful transfer with valid data in *pData.  If error
is detected, it returns a 1 but still waits for a number. */
bool GetData(uchar * pData)
{
    STRING * pS;
    int i;
    bool bError = 0;
    while(1)
    {
        while((i = yylex()) == NTOK);   /* Wait for valid token */
        if(i == ERROR)bError = 1;       /* Error so set flag */
        if(i == NUMERIC)
        {
            pS = &text[1];              /* Set pointer */
            *pData = atoi(pS);
            return bError;
        }
    }
}
/* StartUp initializes the COM port for communication. */
bool StartUp(int Port,int IRQ,uint iBaud,STRING * Protocol)
{
    OpenServer(Port,IRQ,iBaud,Protocol);/* Open COM server */
    ssprintf(BuffA,"@S\n");             /* Enable KPS Signal */
    return(0);
}
/* Test error waits till the CPU is not busy.  It then checks the ERROR flag.
If an ERROR has occured, it sends the message to the PC and returns 1.  If no
error it returns 0 */
bool TestError(STRING * pString,...)
{
    char Buffer[80];
    va_list argptr;
    if(Echo() == ERROR)         /* Send echo and see if an error */
    {
        if(pString)
        {
            va_start(argptr,pString);
            vsprintf(Buffer,pString,argptr);
            printf("ERROR: %s\n",Buffer);
            va_end(argptr);
        }
        return(1);
    }
    return(0);
}

/********** Font Selection **********/
bool SelectFont(uchar xxx)
{
    ssprintf(BuffA,"\nFS%d ",xxx & 0x7);        /* Send string */
    return(TestError("NULL Font"));
}
void SetFontAttribute(uchar ab)
{
    ssprintf(BuffA,"\nFA%d ",ab & 0x3); /* Send string */
}
void SetFontGrayPalet(uchar Palet) 
{
    ssprintf(BuffA,"\nFG%d ",Palet & 0xf);      /* Send string */
}
void SetFontPlane(uchar Plane) 
{
    ssprintf(BuffA,"\nFP%d ",Plane & 0x1);      /* Send string */
}
/* DownLoadFont loads a font from a file into font memory.  It returns a
0 for a successful load.  Returns a zero if an error. sFileName contains the
name of the file to open. */
bool DownLoadFont(STRING * sFileName,uint FontNumber)
{
    FILE * fpFile;
    uchar a;
    uint ModCount = 0;
    uchar Buffer[256];
    uchar * pB;
    bool bEOF = 0;
    bool bError = 0;
    if((fpFile = fopen(sFileName,"rb")) == NULL)
    {
        printf("ERROR: File %s not found\n",sFileName);
        return(1);
    }
    ssprintf(BuffA,"\nFD%d,",(FontNumber & 0x7));
    if(GetFontChar(fpFile,&a))
    {
        fclose(fpFile);
        return(1);
    }
    ssprintf(BuffA,"%d,",a);    /* Send size X */
    if(GetFontChar(fpFile,&a))
    {
        fclose(fpFile);
        return(1);
    }
    ssprintf(BuffA,"%d,",a);    /* Send size Y */
    if(GetFontChar(fpFile,&a))
    {
        fclose(fpFile);
        return(1);
    }
    ssprintf(BuffA,"%d,",a);    /* Send offset */
    if(GetFontChar(fpFile,&a))
    {
        fclose(fpFile);
        return(1);
    }
    ssprintf(BuffA,"%d,",a);    /* Send decenders */
    if(TestError("Down load error"))    /* Check for errors */
    {
        fclose(fpFile);
        return 1;
    }
    while(!bEOF)                /* No error so down load */
    {
        ssprintf(BuffA,"!");    /* Send block transfer character */
        pB = Buffer;            /* Set pointer */
        do      /* Get block of 256 bytes */
        {
            if(!bEOF)
            {
                a = fgetc(fpFile);
                if(feof(fpFile))
                {
                    fclose(fpFile);
                    bEOF = 1;
                    a = 0;
                }
                else if(ferror(fpFile))
                {
                    printf("ERROR: Error reading file %s\n",sFileName);
                    fclose(fpFile);
                    bEOF = 1;
                    bError = 1;
                    a = 0;
                }
                *pB++ = a;
            }
            else
            {
                *pB++ = 0;
            }
        }while(++ModCount % 256);
        SendBlock(Buffer,256);  /* Send block of 256 bytes */
        while(bTXBusy); /* Wait till not busy to protect buffer */
    }
    return bError;
}
bool GetFontChar(FILE * pFile,uchar * pChar)
{
    *pChar = fgetc(pFile);
    if(feof(pFile))
    {
        return(1);
    }
    if(ferror(pFile))
    {
        printf("\nERROR: Error reading file\n");
        return(1);
    }
    return(0);
}

/******** Cursor Positioning ********/
void SetXY(uchar XPos,uchar YPos)
{
    ssprintf(BuffA,"\nCS%d,%d ",XPos,YPos);
}
bool ReadXY(uchar * XPos,uchar * YPos)
{
    ssprintf(BuffA,"\nCG ");
    if(GetData(XPos) || GetData(YPos))
    {
        return(1);
    }
    return(0);
}
void CursorUp(void)
{
    ssprintf(BuffA,"\nCU ");
}
void CursorDown(void)
{
    ssprintf(BuffA,"\nCD ");
}
void CursorLeft(void)
{
    ssprintf(BuffA,"\nCL ");
}
void CursorRight(void)
{
    ssprintf(BuffA,"\nCR ");
}
void SetX(uchar XPos)
{
    ssprintf(BuffA,"\nCX%d ",XPos);
}
void SetY(uchar YPos)
{
    ssprintf(BuffA,"\nCY%d,%d ",YPos);
}
void CursorAttribute(uchar abc)
{
    ssprintf(BuffA,"\nCA%d ",abc & 0x7);
}

/********* Text Configuration *********/
void SetTextWindow(uchar AX,uchar AY,uchar BX,uchar BY)
{
    ssprintf(BuffA,"\nTW%d,%d,%d,%d ",AX,AY,BX,BY);
}
void SetPitch(uchar Pitch)
{
    ssprintf(BuffA,"\nTP%d ",Pitch);
}
void SetHeight(uchar Height)
{
    ssprintf(BuffA,"\nTH%d ",Height);
}

/********* Text Input *********/
/* InputString sends a string to the display */
void InputString(STRING * String)
{
    STRING * pS;
    char * cBuff = (char *)malloc(strlen(String)+1);
    if(!cBuff) return;          /* Return if no memory is allocated. */
    pS = cBuff;
    *pS = 0;
    while(*String)              /* Go till NULL terminator */
    {
        if(*String == '\n')
        {
            if(strlen(cBuff))
            {
                ssprintf(BuffA,"\nIS\"%s\"",cBuff);
                pS = cBuff;
                *pS = 0;
            }
            SetX(0);
            CursorDown();
        }
        else
        {
            *pS++ = *String;
            *pS = 0;
        }
        String++;               /* Move pointer */
    }
    if(strlen(cBuff))
    {
        ssprintf(BuffA,"\nIS\"%s\" ",cBuff);
    }
    free(cBuff);
}

/********* Graphics Input **********/
void DrawBox(uchar AX,uchar AY,uchar BX,uchar BY,uchar TTTF)
{
    ssprintf(BuffA,"\nGX%d,%d,%d,%d,%d ",TTTF & 0xf,AX,AY,BX,BY);
}
void DrawBlock(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT)
{
    ssprintf(BuffA,"\nGB%d,%d,%d,%d,%d ",TT & 0x3,AX,AY,BX,BY);
}
void DrawHoriz(uchar XPos,uchar YPos,char Length,uchar TT)
{
    ssprintf(BuffA,"\nGH%d,%d,%d,%d ",TT & 0x3,XPos,YPos,Length,TT&0x3);
}
void DrawVert(uchar XPos,uchar YPos,char Length,uchar TT)
{
    ssprintf(BuffA,"\nGV%d,%d,%d,%d ",TT & 0x3,XPos,YPos,Length,TT&0x3);
}
void DrawVector(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT)
{
    ssprintf(BuffA,"\nGC%d,%d,%d,%d,%d ",TT & 0x3,AX,AY,BX,BY);
}
void SetPixel(uchar XPos,uchar YPos,uchar TT)
{
    ssprintf(BuffA,"\nGI%d,%d,%d,%d ",TT & 0x3,XPos,YPos,TT&0x3);
}
void SetGrayPalet(uchar Palet) 
{
    ssprintf(BuffA,"\nGG%d ",Palet & 0xf);
}
void SetPlane(uchar Plane)
{
    ssprintf(BuffA,"\nGP%d ",Plane & 0x1);
}

/******* Button Input *********/

/* PlaceButton return 0 if putton placement was successful.  Returns 1 if error
and button cound not be placed. */
bool PlaceButton(uchar Code,uchar Col,uchar Row)
{
    ssprintf(BuffA,"\nBC%d,%d",Code,((Row<<4)&0x70)|(Col&0xf));
    return(TestError(NULLP));
}
void LoadButtonBuffer(STRING * String)
{
    ssprintf(BuffA,"\nBL\"%s\" ",String);
}
/* Return 0 if no error, 1 if error */
bool GetButtonSize(uchar KeyCode,uchar * RData)
{
    ssprintf(BuffA,"\nBS%d ",KeyCode);
    return(GetData(RData));             /* Get Data */
}
bool PlacePhantomButton(uchar KeyCode,uchar Col,uchar Row,uchar Length)
{
    ssprintf(BuffA,"\nBH%d,%d,%d ",KeyCode,(((Row<<4)&0x70)|(Col&0xf)),Length);
    return(TestError(NULLP));
}
bool DeleteButton(uchar KeyCode)
{
    ssprintf(BuffA,"\nBD%d ",KeyCode);
    return(TestError("Button not present to delete"));
}
void DeleteAllButtons(void)
{
    ssprintf(BuffA,"\nBE ");
}
uchar ReadKeyCode(void)
{
    uchar c;
    ssprintf(BuffA,"\nBK ");
    GetData(&c);
    return(c);
}
void SetButtonAttribute(uchar abc) 
{
    ssprintf(BuffA,"\nBA%d ",(abc & 0x07));
}
void SetButtonGrayPalet(uchar Palet) 
{
    ssprintf(BuffA,"\nBG%d ",(Palet & 0x0f));
}
void SetButtonPlane(uchar Plane)
{
    ssprintf(BuffA,"\nBP%d ",(Plane & 0x01));
}

/********* Display Control *********/

void BlankDisplay(void)
{
    ssprintf(BuffA,"\nDB ");
}
void ClearDisplay(void)
{
    ssprintf(BuffA,"\nDC ");
}
void Refresh(void)
{
    return;
}
void SetAutoRefresh(uchar na)
{
    return;
}
/* DumpDisplayRAM dumps the current display to file named sFileName */
bool DumpDisplayRAM(STRING * na)
{
    return(0);
}
bool LoadDisplayRAM(STRING * na)
{
    return(0);
}
void MoveBlockVert(uchar AX,uchar AY,uchar BX,uchar BY,char Distance)
{
    ssprintf(BuffA,"\nDV%d,%d,%d,%d,%d ",AX,AY,BX,BY,Distance);
}
void MoveBlockHoriz(uchar AX,uchar AY,uchar BX,uchar BY,char Distance)
{
    ssprintf(BuffA,"\nDH%d,%d,%d,%d,%d ",AX,AY,BX,BY,Distance);
}
void LoadGrayDither(uchar GD1,uchar GD2)
{
    ssprintf(BuffA,"\nDD%d,%d ",GD1,GD2);
}
void ReverseVideo(uchar a)
{
    ssprintf(BuffA,"\nDR%d ",a & 0x01);
}

/******** System Instructions *********/
void SoftReset(void)
{
    ssprintf(BuffA,"\nSR ");
}
void SetContrast(uchar Data)
{
    ssprintf(BuffA,"\nSC%d ",Data);
}
void SetEL(uchar a)
{
    ssprintf(BuffA,"\nSE%d ",a & 0x01);
}
void NOP(void)
{
    ssprintf(BuffA,"\nSN ");
}
void SetBeeper(uchar a)
{
    ssprintf(BuffA,"\nSB%d ",a & 0x01);
}
bool ReadKeyMatrix(uchar * RData)
{
    bool bError=0;
    uint i;
    ssprintf(BuffA,"\nSK ");
    for(i=0;(i<6)&&!bError;i++)
    {
        bError = GetData(RData);        /* Get data */
        ++RData;                        /* Incriment pointer */
    }
    if(bError)
    {
        printf("ERROR: Reading key matrix\n");
        return(1);
    }
    return(0);
}
bool ReadAuxPort(uchar * pAuxData)
{
    ssprintf(BuffA,"\nSA ");
    if(GetData(pAuxData))
    {
        return(1);
    }
    return(0);
}
void WriteAuxPort(uchar AuxData)
{
    ssprintf(BuffA,"\nSW%d ",AuxData);
}
